'use strict';

/* For ESLint: List any global identifiers used in this file below */
/* global chrome, STATS, openTab, recordErrorMessage, log, getSettings */

// if the ping reponse indicates a survey (tab or overlay)
// gracefully processes the request
const SURVEY = (function surveyConstructor() {
  // Only allow one survey per browser startup, to make sure users don't get
  // spammed due to bugs in AdBlock / the ping server / the browser.
  let surveyAllowed = true;

  // Double check that the survey should be shown
  // Used by both Tab and Overlay Surveys
  // Inputs:
  //   surveyData: JSON survey information from ping server
  //   callback(): called with no arguments if the survey should be shown
  const shouldShowSurvey = function (surveyData, callback) {
    // Check if we should show survey only if it can actually be shown
    // based on surveyAllowed.
    if (surveyAllowed) {
      const data = { cmd: 'survey', u: STATS.userId(), sid: surveyData.survey_id };
      $.post(STATS.statsUrl, data, (responseData) => {
        let resData;
        try {
          resData = JSON.parse(responseData);
        } catch (e) {
          log('Error parsing JSON: ', responseData, ' Error: ', e);
        }

        if (resData && resData.should_survey === 'true') {
          surveyAllowed = false;
          callback();
        }
      });
    }
  };

  // open a Tab for a full page survey
  const processTab = function (surveyData) {
    const waitForUserAction = function () {
      chrome.tabs.onCreated.removeListener(waitForUserAction);
      const openTabIfAllowed = function () {
        shouldShowSurvey(surveyData, () => {
          openTab(`https://adblockprotect.com/${surveyData.open_this_url}`, true);
        });
      };

      openTabIfAllowed();
    };

    if (chrome.tabs.onCreated.hasListener(waitForUserAction)) {
      chrome.tabs.onCreated.removeListener(waitForUserAction);
    }

    chrome.tabs.onCreated.addListener(waitForUserAction);
  }; // end of processTab()

  // Check the response from a ping to see if it contains valid survey instructions.
  // If so, return an object containing data about the survey to show.
  // Otherwise, return null.
  // Used by both Tab and Overlay Surveys
  // Inputs:
  //   responseData: string response from a ping
  const surveyDataFrom = function (responseData) {
    if (!responseData || responseData.length === 0 || responseData.trim().length === 0) {
      return null;
    }

    log('validating ping response data', responseData);

    let surveyData;
    try {
      surveyData = JSON.parse(responseData);
      if (!surveyData.open_this_url
            || !surveyData.open_this_url.match
            || !surveyData.open_this_url.match(/^\/survey\//)) {
        log('bad survey data', responseData);
        return null;
      }
    } catch (e) {
      log('Something went wrong with parsing survey data.');
      log('error', e);
      log('response data', responseData);
      return null;
    }

    return surveyData;
  };

  return {
    maybeSurvey(responseData) {
      if (getSettings().show_survey === false) {
        return;
      }

      const surveyData = surveyDataFrom(responseData);
      if (!surveyData) {
        return;
      }
      if (surveyData.type === 'tab') {
        processTab(surveyData);
      }
    }, // end of maybeSurvey

    types() {
      // 'O' = Overlay Surveys and 'T' = Tab Surveys
      return 'T';
    },
  };
}());
