// Inspired by the DuckDuckGo Privacy Extension (https://github.com/duckduckgo/duckduckgo-privacy-extension)
// Original source for this file can be found here:
// https://github.com/duckduckgo/duckduckgo-privacy-extension/tree/develop/shared/js/background/classes

/*
 * Copyright (C) 2012, 2016 DuckDuckGo, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* For ESLint: List any global identifiers used in this file below */
/* global require log module chrome blockCounts exports Buffer */

'use strict';

/**
 *
 */
const tldjs = require('tldjs');
const DDGsettings = require('../settings.es6');
const utils = require('../utils.es6');
const abpLists = require('../abp-lists.es6');
const browserWrapper = require('../chrome-wrapper.es6');

class Site {
  constructor(url) {
    this.url = url || '';

    let domain = utils.extractHostFromURL(this.url) || '';
    domain = domain.toLowerCase();

    this.domain = domain;
    this.whitelisted = false; // user-whitelisted sites; applies to all privacy features
    this.setWhitelistStatusFromGlobal(domain);
    this.isBroken = Site.checkBrokenSites(domain); // broken sites reported to github repo
    this.didIncrementCompaniesData = false;

    this.parentEntity = utils.findParent(domain) || '';
    // set specialDomainName when the site is created
    this.specialDomainName = this.getSpecialDomain();
  }

  /*
     * check to see if this is a broken site reported on github
    */
  static checkBrokenSites(domainParam) {
    const domain = domainParam;
    const trackersWhitelistTemporary = abpLists.getTemporaryWhitelist();

    if (!trackersWhitelistTemporary) {
      return undefined;
    }

    const parsedDomain = tldjs.parse(domain);
    const hostname = parsedDomain.hostname || domain;

    // If root domain in temp whitelist, return true
    return trackersWhitelistTemporary.some(brokenSiteDomain => hostname.match(new RegExp(`${brokenSiteDomain}$`)));
  }

  /*
     * When site objects are created we check the stored whitelists
     * and set the new site whitelist statuses
     */
  setWhitelistStatusFromGlobal() {
    const globalwhitelists = ['whitelisted'];
    globalwhitelists.forEach((name) => {
      const list = DDGsettings.getSetting(name) || {};
      this.setWhitelisted(name, list[this.domain]);
    });
  }

  setWhitelisted(name, value) {
    this[name] = value;
  }

  /*
     * Send message to the popup to rerender the whitelist
     */
  static notifyWhitelistChanged() {
    chrome.runtime.sendMessage({ whitelistChanged: true });
  }

  isWhiteListed() {
    return this.whitelisted;
  }


  /*
     * specialDomain
     *
     * determine if domain is a special page
     *
     * returns: a useable special page description string.
     *          or null if not a special page.
     */
  getSpecialDomain() {
    const extensionId = browserWrapper.getExtensionId();
    const url = this.url;
    const localhostName = 'localhost';
    let domain = this.domain;

    if (url === '') {
      return 'new tab';
    }

    // Both 'localhost' and the loopback ip have to be specified
    // since they're treated as different domains
    if (domain === localhostName || domain.match(/^127\.0\.0\.1/)) {
      return localhostName;
    }

    // Handle non-routable meta-address
    if (domain.match(/^0\.0\.0\.0/)) {
      return domain;
    }

    // for some reason chrome passes this back from webNavigation events
    // for new tabs instead of chrome://newtab
    //
    // "local-ntp" -> "local new tab page"
    if (url.match(/^chrome-search:\/\/local-ntp/)) {
      return 'new tab';
    }

    // for special pages with a protocol, just return whatever
    // word comes after the protocol
    // e.g. 'chrome://extensions' -> 'extensions'
    if (url.match(/^chrome:\/\//)
                || url.match(/^vivaldi:\/\//)) {
      if (domain === 'newtab') {
        domain = 'new tab';
      }

      return domain;
    }

    // FF-style about: pages don't get their domains parsed properly
    // so just extract the bit after about:
    if (url.match(/^about:/)) {
      /* eslint-disable-next-line prefer-destructuring */
      domain = url.match(/^about:([a-z-]+)/)[1];
      return domain;
    }

    // extension pages
    if (url.match(/^(chrome|moz)-extension:\/\//)) {
      // this is our own extension, let's try and get a meaningful description
      if (domain === extensionId) {
        const matches = url.match(/^(?:chrome|moz)-extension:\/\/[^/]+\/html\/([a-z-]+).html/);

        if (matches && matches[1]) {
          return matches[1];
        }
      }

      // if we failed, or this is not our extension, return a generic message
      return 'extension page';
    }

    return null;
  }
}
//
module.exports = Site;
