// Inspired by the DuckDuckGo Privacy Extension (https://github.com/duckduckgo/duckduckgo-privacy-extension)
// Original source for this file can be found here:
// https://github.com/duckduckgo/duckduckgo-privacy-extension/tree/develop/shared/js/background

/*
 * Copyright (C) 2012, 2016 DuckDuckGo, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* For ESLint: List any global identifiers used in this file below */
/* global require log module chrome blockCounts exports */

'use strict';

const tldjs = require('tldjs');
const entityMap = require('./entityMap');
const constants = require('./constants');
const parseUserAgentString = require('./parse-user-agent-string.es6');

const browserInfo = parseUserAgentString();

function extractHostFromURL(url, shouldKeepWWW) {
  if (!url) {
    return '';
  }

  const urlObj = tldjs.parse(url);
  let hostname = urlObj.hostname || '';

  if (!shouldKeepWWW) {
    hostname = hostname.replace(/^www\./, '');
  }

  return hostname;
}

function extractTopSubdomainFromHost(host) {
  if (typeof host !== 'string') {
    return false;
  }
  const rgx = /\./g;
  if (host.match(rgx) && host.match(rgx).length > 1) {
    return host.split('.')[0];
  }
  return false;
}

// pull off subdomains and look for parent companies
function findParent(url) {
  const parts = extractHostFromURL(url).split('.');

  while (parts.length > 1) {
    const joinURL = parts.join('.');

    if (entityMap[joinURL]) {
      return entityMap[joinURL];
    }
    parts.shift();
  }
  return undefined;
}

function getProtocol(url) {
  const a = document.createElement('a');
  a.href = url;
  return a.protocol;
}

function getCurrentURL(callback) {
  chrome.tabs.query({ active: true, lastFocusedWindow: true }, (tabData) => {
    if (tabData.length) {
      callback(tabData[0].url);
    }
  });
}

function getCurrentTab() {
  return new Promise((resolve) => {
    chrome.tabs.query({ active: true, lastFocusedWindow: true }, (tabData) => {
      if (tabData.length) {
        resolve(tabData[0]);
      }
    });
  });
}

// Browser / Version detection
// Get correct name for fetching UI assets
function getBrowserName() {
  if (!browserInfo || !browserInfo.browser) {
    return null;
  }

  let browser = browserInfo.browser.toLowerCase();
  if (browser === 'firefox') {
    browser = 'moz';
  }

  return browser;
}

// Determine if upgradeToSecure supported (Firefox 59+)
function getUpgradeToSecureSupport() {
  let canUpgrade = false;
  if (getBrowserName() !== 'moz') {
    return canUpgrade;
  }

  if (browserInfo && browserInfo.version >= 59) {
    canUpgrade = true;
  }

  return canUpgrade;
}

// Chrome errors with 'beacon', but supports 'ping'
// Firefox only blocks 'beacon' (even though it should support 'ping')
function getBeaconName() {
  const beaconNamesByBrowser = {
    chrome: 'ping',
    moz: 'beacon',
  };

  return beaconNamesByBrowser[getBrowserName()];
}

// Return requestListenerTypes + beacon or ping
function getUpdatedRequestListenerTypes() {
  const requestListenerTypes = constants.requestListenerTypes.slice();
  requestListenerTypes.push(getBeaconName());

  return requestListenerTypes;
}

module.exports = {
  extractHostFromURL,
  extractTopSubdomainFromHost,
  getCurrentURL,
  getCurrentTab,
  getProtocol,
  getBrowserName,
  getUpgradeToSecureSupport,
  findParent,
  getBeaconName,
  getUpdatedRequestListenerTypes,
};
